import { Duration, asyncMap, asyncZip, enforce_nonnull, pause } from "vscript";
import { get_peaks } from "./audio.js";
function add_db(dbs) {
    return (10 *
        Math.log10(dbs.reduce((accu, currval) => {
            return accu + Math.pow(10, currval / 10);
        }, 0)));
}
export class AutoMixer {
    promise;
    _mixer;
    updateInterval = new Duration(20, "ms");
    get mixer() {
        return this._mixer;
    }
    enable = true;
    constructor(mixer) {
        this._mixer = mixer;
        this.promise = this.mix();
    }
    async start() {
        if (this.enable && this.promise)
            return;
        this.enable = true;
        this.promise = this.mix();
    }
    async stop() {
        this.enable = false;
        await this.promise;
        this.promise = null;
    }
    async mix() {
        const get_single_ppm = async (channel) => {
            if (!(!!channel && !!channel.enclosing_subtree))
                return -1024;
            const audio_essence = channel.enclosing_subtree;
            const peaks = await get_peaks(audio_essence);
            return (peaks[channel.index] ?? -1024 * 10) / 10;
        };
        const caps = enforce_nonnull(await this.mixer.capabilities.status.read());
        while (this.enable) {
            const inputs = (await this.mixer.inputs.read()).slice(0, caps.channels);
            const levels = await asyncMap(inputs, async (inp) => {
                return await get_single_ppm(inp);
            });
            const sum_levels = add_db(levels);
            await asyncZip(levels, await this.mixer.faders.rows(), async (in_level, fader, idx) => {
                if (idx >= caps?.channels)
                    return;
                const target_gain = Math.min(in_level - sum_levels, 12.0);
                await fader.gain.write(target_gain);
            });
            await pause(this.updateInterval);
        }
    }
}
